#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   tagHelper.py
@Time    :   2019/07/18
@Author  :   Yaronzz 
@Version :   2.0
@Contact :   yaronhuang@foxmail.com
@Desc    :   
'''
import os
import sys
import requests
from mutagen import File
from mutagen import flac
from mutagen import mp4
from mutagen.id3 import TALB, TCOP, TDRC, TIT2, TPE1, TRCK, APIC, TCON, TCOM, TSRC
from aigpy import pathHelper
import aigpy.netHelper as netHelper


def __getHash__(pHash, key):
    if key in pHash:
        return pHash[key]
    return ''


def __lower__(inputs):
    if isinstance(inputs, str):
        inputs = inputs.decode('utf-8')
    inputs = inputs.lower().encode('utf-8')
    return inputs


def __getExtension__(filepath):
    index = filepath.rfind('.')
    ret = filepath[index + 1:len(filepath)]
    v = sys.version_info
    if v[0] > 2:
        return str.lower(ret)
    else:
        return __lower__(ret)


def __getFileData__(filepath):
    if 'http' in filepath:
        re = requests.get(filepath)
        return re.content
    try:
        with open(filepath, "rb") as f:
            data = f.read()
            return data
    except:
        return None


def __tryInt__(obj):
    try:
        ret = int(obj)
        return ret
    except:
        return 0


def __getArrayStr__(array):
    if array is None:
        return ''
    if len(array) <= 0:
        return array
    ret = None
    for item in array:
        if ret is None:
            ret = item
            continue
        ret += ', ' + item
    return ret


def __noneToEmptyString__(obj):
    if obj is None:
        return ''
    else:
        return obj


class TagTool(object):
    def __init__(self, filePath):
        if os.path.isfile(filePath) is False:
            return

        self._filepath = filePath
        self._ext = __getExtension__(filePath)
        self._handle = File(filePath)

        self.title = ''
        self.album = ''
        self.albumartist = ''
        self.artist = ''
        self.copyright = ''
        self.tracknumber = ''
        self.totaltrack = ''
        self.discnumber = ''
        self.totaldisc = ''
        self.genre = ''
        self.date = ''
        self.composer = ''
        self.isrc = ''

    def save(self, coverPath=None):
        try:
            if 'mp3' in self._ext:
                return self.__saveMp3__(coverPath)
            if 'flac' in self._ext:
                return self.__saveFlac__(coverPath)
            if 'mp4' in self._ext or 'm4a' in self._ext:
                return self.__saveMp4__(coverPath)
            return False
        except:
            return False

    def __saveMp3__(self, coverPath):
        self._handle.tags.add(TIT2(encoding=3, text=self.title))
        self._handle.tags.add(TALB(encoding=3, text=self.album))
        # self._handle.tags.add(TOPE(encoding=3, text=self.albumartist))
        self._handle.tags.add(TPE1(encoding=3, text=self.artist))
        self._handle.tags.add(TCOP(encoding=3, text=self.copyright))
        self._handle.tags.add(TRCK(encoding=3, text=str(self.tracknumber)))
        # self._handle.tags.add(TRCK(encoding=3, text=self.discnum))
        self._handle.tags.add(TCON(encoding=3, text=self.genre))
        self._handle.tags.add(TDRC(encoding=3, text=self.date))
        self._handle.tags.add(TCOM(encoding=3, text=self.composer))
        self._handle.tags.add(TSRC(encoding=3, text=self.isrc))
        self.__savePic__(coverPath)
        self._handle.save()
        return True

    def __saveFlac__(self, coverPath):
        if self._handle.tags is None:
            self._handle.add_tags()
        self._handle.tags['title'] = self.title
        self._handle.tags['album'] = self.album
        self._handle.tags['albumartist'] = self.albumartist
        self._handle.tags['artist'] = self.artist
        self._handle.tags['copyright'] = __noneToEmptyString__(self.copyright)
        self._handle.tags['tracknumber'] = str(self.tracknumber)
        self._handle.tags['tracktotal'] = str(self.totaltrack)
        self._handle.tags['discnumber'] = str(self.discnumber)
        self._handle.tags['disctotal'] = str(self.totaldisc)
        self._handle.tags['genre'] = __noneToEmptyString__(self.genre)
        self._handle.tags['date'] = __noneToEmptyString__(self.date)
        self._handle.tags['composer'] = __noneToEmptyString__(self.composer)
        self._handle.tags['isrc'] = str(self.isrc)
        self.__savePic__(coverPath)
        self._handle.save()
        return True

    def __saveMp4__(self, coverPath):
        self._handle.tags['\xa9nam'] = self.title
        self._handle.tags['\xa9alb'] = self.album
        self._handle.tags['aART'] = __getArrayStr__(self.albumartist)
        self._handle.tags['\xa9ART'] = __getArrayStr__(self.artist)
        self._handle.tags['cprt'] = __noneToEmptyString__(self.copyright)
        self._handle.tags['trkn'] = [[__tryInt__(self.tracknumber), __tryInt__(self.totaltrack)]]
        self._handle.tags['disk'] = [[__tryInt__(self.discnumber), __tryInt__(self.totaldisc)]]
        self._handle.tags['\xa9gen'] = __noneToEmptyString__(self.genre)
        self._handle.tags['\xa9day'] = __noneToEmptyString__(self.date)
        self._handle.tags['\xa9wrt'] = __getArrayStr__(self.composer)
        self.__savePic__(coverPath)
        self._handle.save()
        return True

    def __savePic__(self, coverPath):
        data = __getFileData__(coverPath)
        if data is None:
            return
        if 'flac' in self._ext:
            pic = flac.Picture()
            pic.data = data
            if '.jpg' in coverPath:
                pic.mime = u"image/jpeg"
            self._handle.clear_pictures()
            self._handle.add_picture(pic)
        if 'mp3' in self._ext:
            self._handle.tags.add(APIC(encoding=3, data=data))
        if 'mp4' in self._ext or 'm4a' in self._ext:
            pic = mp4.MP4Cover(data)
            self._handle.tags['covr'] = [pic]

# test = TagTool('e:\\1.m4a')
# test.album = ['ff']
# test.albumartist = ['yaron', 'f']
# test.artist = ['huang', 'dd']
# test.copyright = None
# test.title = 'yes'
# test.tracknumber = '1'
# test.genre = 'fdsa'
# test.date = '2019-2-3'
# test.save('e:\\1.jpg')
# y =str.lower("Ye")

# t = str("Ye").lower()
# a = o



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   jsonHelper.py
@Time    :   2020/08/10
@Author  :   Yaronzz
@Version :   2.0
@Contact :   yaronhuang@foxmail.com
@Desc    :   
'''
import json
from aigpy.dictHelper import DictTool


def modelToDict(model):
    if model == None:
        return None
    pr = {}
    for name in dir(model):
        value = getattr(model, name)
        if name[0] == '_':
            continue
        if callable(value):
            continue
        pr[name] = value
    return pr

def dictToModel(indict, model):
    if indict == None or model == None:
        return None
    ret = model.__class__()
    maps = DictTool(indict)

    for key in dir(ret):

        if key[0] == '_':
            continue
        if key.lower() not in maps:
            if __isObject__(getattr(ret, key)):
                setattr(ret, key, None)
            continue

        # 判断是否为字典数组
        lvalue = maps[key.lower()]
        if __isDictList__(lvalue):
            value = dictListToModelList(lvalue, getattr(ret, key))
        # 判断是否为字典
        elif __isDict__(lvalue):
            value = dictToModel(lvalue, getattr(ret, key))
        else:
            value = lvalue

        setattr(ret, key, value)
    return ret


def dictListToModelList(jList, model):
    if jList == None or model == None:
        return jList
    ret = []
    for item in jList:
        data = dictToModel(item, model)
        ret.append(data)
    return ret


def __isDict__(data):
    if isinstance(data, dict):
        return True
    return False


def __isObject__(data):
    if isinstance(data, object):
        return True
    return False


def __isDictList__(data):
    if isinstance(data, list):
        for item in data:
            return __isDict__(item)
    return False



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

import os
import tarfile
import zipfile


def __getParaType__(para):
    '''
    @return
        0: file path
        1: dir path
        2: file paths
    '''
    if os.path.isfile(para):
        return 0
    if os.path.isdir(para):
        return 1
    return 2

def __getZipType__(zipName):
    '''
    @return
        tar
        zip
    '''
    name = os.path.basename(zipName)
    if name.lower().find('.tar') > 0:
        return 'tar'
    else:
        return 'zip'

def __open__(zipName, zipType, mode = 'w'):
    if zipType == 'tar':
        return tarfile.open(zipName, mode)
    else:
        return zipfile.ZipFile(zipName, mode, zipfile.ZIP_DEFLATED)

def __write__(pZip, zipType, filename, parcname):
    if zipType == 'tar':
        pZip.add(filename, arcname=parcname)
    else:
        pZip.write(filename, arcname=parcname)


def zip(inPath, outPath):
    '''zip files or path

    @param
        inPath: file path/file paths/dir
        output: zip file pathname
    @return
        True/False
    '''
    try:
        paraType = __getParaType__(inPath)
        zipType = __getZipType__(outPath)
        pZip = __open__(outPath, zipType)

        if paraType == 2:
            for file in inPath:
                pZip.__write__(pZip, zipType, file, os.path.basename(file))

        if paraType == 0:
            pZip.__write__(pZip, zipType, inPath, os.path.basename(inPath))

        if paraType == 1:
            name = os.path.dirname(inPath)
            for dirpath, dirnames, filenames in os.walk(inPath):
                path = dirpath.replace(name, '')
                path = path and path + os.sep or ''
                for filename in filenames:
                    pZip.__write__(pZip, zipType, os.path.join(dirpath, filename), path + filename)

        pZip.close()
        return True
    except:
        return False


def unzip(zipName, outPath):
    zipType = __getZipType__(zipName)
    try:
        pZip = __open__(zipName, zipType, 'r')
        pZip.extractall(path = outPath)
        pZip.close()
        return True
    except:
        return False



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   dictHelper.py
@Time    :   2020/08/14
@Author  :   Yaronzz
@Version :   2.0
@Contact :   yaronhuang@foxmail.com
@Desc    :   
'''


class DictTool(dict):
    def __init__(self, **kw):
        self.kp = {}
        for k in kw.keys():
            self.__setitem__(k, kw[k])

    def __init__(self, kw):
        self.kp = {}
        for k in kw:
            self.__setitem__(k, kw[k])

    def __contains__(self, k):
        if isinstance(k, str):
            kn = k.lower()
            if not self.kp.__contains__(kn):
                return False
            k = kn
        return super().__contains__(k)

    def __setitem__(self, k, v):
        if isinstance(k, str):
            kn = k.lower()
            self.kp[kn] = k
            k = kn
        super().__setitem__(k, v)

    def __delitem__(self, k):
        if isinstance(k, str):
            k = k.lower()
            self.kp.pop(k)
        super.__delitem__(k)

    def __getitem__(self, k):
        if isinstance(k, str):
            k = k.lower()
        return super().__getitem__(k)

    def actual_key_case(self, k):
        if isinstance(k, str):
            return self.kp[k.lower()]
        return k



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   pipHelper.py
@Time    :   2019/03/11
@Author  :   Yaronzz 
@Version :   2.0
@Contact :   yaronhuang@foxmail.com
@Desc    :   pip server tool
'''

import aigpy.netHelper as netHelper
import sys
import subprocess
from aigpy import fileHelper, pathHelper
import re

def getInfo(projectName):
    """Get project information from pypi
    - Return: json or None                              
    """
    url = 'https://pypi.org/pypi/' + projectName + '/json'
    ret = netHelper.downloadJson(url,None)
    return ret
    
def getLastVersion(projectName):
    """Get project version from pypi
    - Return: str or None                              
    """
    try:
        ret = getInfo(projectName)
        if ret is None:
            return None
        return ret['info']['version']
    except:
        return None

def getVersionList(projectName):
    """Get project all versions from pypi
    - Return: json or None                              
    """
    try:
        ret = getInfo(projectName)
        if ret is None:
            return None
        return ret['releases']
    except:
        return None

def getInstalledVersion(projectName, pipver='3'):
    try:
        cmd = 'pip' + pipver + ' freeze'
        stdoutFile = 'piplibversion-stdout.txt'
        fp = open(stdoutFile, 'w')
        res = subprocess.call(cmd, shell=True, stdout=fp, stderr=fp)
        fp.close()
        txt = fileHelper.getFileContent(stdoutFile)

        lines = txt.split('\n')
        for item in lines:
            if item.find(projectName + '==') == 0:
                version = item[len((projectName + '==')):]
                return version
    except Exception as e:
        pass
    return ''



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

import aigpy.configHelper as ConfigHelper


def cmpVersion(ver1, ver2):
    array1 = ver1.split('.')
    array2 = ver2.split('.')
    iIndex = 0
    for obj in array1:
        if len(array2) <= iIndex:
            break
        if obj > array2[iIndex]:
            return 1
        if obj < array2[iIndex]:
            return -1
        iIndex = iIndex + 1
    return 0


class VersionFile(object):
    def __init__(self, path=None):
        self.version      = None
        self.mainFile     = None
        self.elseFileList = []
        self.isZip        = 0
        self.zipFile      = ''
        if path != None:
            self.readFile(path)

    def saveFile(self, path):
        if path is None or self.version is None or self.mainFile is None:
            return False
        if self.isZip != 0 and self.zipFile == '':
            return False

        check = ConfigHelper.SetValue('common', 'version',  self.version, path)
        check = ConfigHelper.SetValue('common', 'mainfile', self.mainFile, path)
        if check is False:
            return False
        check = ConfigHelper.SetValue('common', 'iszip',  self.isZip, path)
        check = ConfigHelper.SetValue('common', 'zipfile',  self.isZip, path)

        if self.elseFileList is None or len(self.elseFileList) == 0:
            return True
        ConfigHelper.SetValue('common', 'elsenum', len(self.elseFileList), path)
        index = 0
        for item in self.elseFileList:
            ConfigHelper.SetValue('common', 'else' + index, item, path)
            index = index + 1
        return True

    def readFile(self, path):
        if path is None:
            return False
        ver = ConfigHelper.GetValue('common', 'version', '', path)
        mainFile = ConfigHelper.GetValue('common', 'mainfile', '', path)
        if ver == '' or mainFile == '':
            return False

        isZip = ConfigHelper.GetValue('common', 'iszip', 0, path)
        isZip = int(isZip)
        zipFile = ConfigHelper.GetValue('common', 'zipfile', '', path)
        if isZip != 0 or zipFile == '':
            return False

        elseNum = ConfigHelper.GetValue('common', 'elsenum', 0, path)
        elseNum = int(elseNum)
        elseList = []
        index = 0
        if elseNum > 0:
            obj = ConfigHelper.GetValue('common', 'else' + index, '', path)
            index = index + 1
            elseList.append(obj)

        self.version = ver
        self.mainFile = mainFile
        self.elseFileList = elseList
        self.isZip = isZip
        self.zipFile = zipFile
        return True






#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   fileHelper.py
@Time    :   2019/03/11
@Author  :   Yaronzz 
@Version :   2.0
@Contact :   yaronhuang@foxmail.com
@Desc    :   
'''

import os


def getFileSize(path):
    try:
        if os.path.isfile(path) is False:
            return 0
        return os.path.getsize(path)
    except:
        return 0


def getFileContent(path, isBin=False):
    mode = 'r'
    if isBin:
        mode = 'rb'
    try:
        size = getFileSize(path)
        if size <= 0:
            return ""
        with open(path, mode) as fd:
            content = fd.read(size)
        return content
    except:
        return ""

def getFileLines(path):
    content = getFileContent(path)
    if content == "":
        return []
    lines = content.split('\n')
    return lines

def write(path, content, mode):
    try:
        with open(path, mode) as fd:
            fd.write(content)
        return True
    except Exception as e:
        return False

def writeLines(path, lines:list, mode):
    content = ""
    for item in list:
        content += item + '\n'
    return write(path, content, mode)




#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   m3u8Helper.py
@Time    :   2019/08/23
@Author  :   Yaronzz 
@Version :   2.0
@Contact :   yaronhuang@foxmail.com
@Desc    :   
'''
import re
import shutil
from aigpy import netHelper
from aigpy.progressHelper import ProgressTool
from aigpy.pathHelper import getDirName, getDiffTmpPathName, mkdirs
from aigpy.threadHelper import ThreadTool
from aigpy.fileHelper import getFileContent

def paresUrl(url):
    '''Get ts-urls from m3u8-url'''
    content = netHelper.downloadString(url, None)
    pattern = re.compile(r"(?<=http).+?(?=\\n)")
    plist   = pattern.findall(str(content))
    urllist = []
    for item in plist:
        urllist.append("http"+item)
    return urllist

def __threadfunc__(url, filepath, progress):
    retrycount = 3
    try:
        while retrycount > 0:
            retrycount = retrycount - 1
            check = netHelper.downloadFile(url, filepath, 30)
            if check:
                break
    except:
        pass
    progress.step()


def __merger__(files, filepath):
    try:
        with open(filepath, "wb") as fd:
            for item in files:
                data = getFileContent(item, True)
                fd.write(data)
        return True
    except:
        return False
    

def download(url, descpath, threadnum = 15):
    '''Download file by m3u8-url'''
    try:
        urllist = paresUrl(url)
        if len(urllist) <= 0:
            return False
        
        threads = ThreadTool(threadnum)

        # Creat tmpdir
        path = getDirName(descpath)
        tmpPath = getDiffTmpPathName(path)
        if mkdirs(tmpPath) is False:
            return False

        # Progress
        progress = ProgressTool(len(urllist), 20)

        # Download files
        files = []
        for i, item in enumerate(urllist):
            filepath = tmpPath + '/' + str(i) + '.ts'
            files.append(filepath)
            threads.start(__threadfunc__, item, filepath, progress)
        threads.waitAll()

        # merger
        __merger__(files, descpath)
        shutil.rmtree(tmpPath)
        threads.close()
        return True
    except:
        shutil.rmtree(tmpPath)
        return False





#!/usr/bin/env python
# -*- encoding: utf-8 -*-


'''
@File    :   configHelper.py
@Time    :   2018/12/17
@Author  :   Yaronzz 
@Version :   2.0
@Contact :   yaronhuang@foxmail.com
@Desc    :   Config Tool
'''
import os
import configparser
from binascii import b2a_hex, a2b_hex
from Crypto.Cipher import AES

def Count(fileName, section=None):
    """Get para number"""
    try:
        ret = 0
        cf = configparser.ConfigParser()
        cf.read(fileName)
        if section is None:
            seclist = cf.sections()
            for sec in seclist:
                oplist = cf.options(sec)
                ret    = ret + len(oplist)
        elif cf.has_section(section):
            ret = len(cf[section])
        return ret
    except:
        return 0

def Sections(fileName):
    """Get groups"""
    try:
        cf = configparser.ConfigParser()
        cf.read(fileName)
        return cf.sections()
    except:
        return None


def GetValue(section, key, default, fileName, aesKey=None):
    try:
        cf = configparser.ConfigParser()
        cf.read(fileName)
        if not cf.has_section(section):
            return default
        
        if key in cf[section]:
            default = cf.get(section, key)

        if aesKey is not None:
            if default is not None and len(default) > 0:
                func = AES_FUNC(aesKey)
                default = func.decrypt(default)
        return default
    except:
        return default


def SetValue(section, key, value, fileName, aesKey=None):
    try:
        if os.access(fileName, 0) is False:
            fp = open(fileName, "w")
            fp.close()

        cf = configparser.ConfigParser()
        cf.read(fileName)
        if cf.has_section(section) is False:
            cf[section] = {}

        real_value = value
        if aesKey is not None:
            if value is not None and len(value) > 0:
                func = AES_FUNC(aesKey)
                real_value = func.encrypt(value)
                real_value = str(real_value, encoding="utf-8")

        cf[section][key] = real_value
        with open(fileName, "w") as f:
            cf.write(f)
        return True
    except:
        return False


def ParseNoEqual(fileName):
    ret = {}
    try:
        fd  = open(fileName, 'r')
        arr = fd.readlines()
        group = None
        for item in arr:
            item = item.strip()
            if len(item) <= 0:
                continue
            if item[0] == '#':
                continue
            elif item[0] == '[' and item[len(item) - 1] == ']':
                group = item[1:len(item) - 1]
                ret[group] = []
            elif group is None:
                continue
            else:
                ret[group].append(item)
        return ret     
    except:
        return ret

class AES_FUNC():
    def __init__(self, key):
        self.key = key
        self.mode = AES.MODE_ECB
        self.AES_LENGTH = 16
        self.cryptor = AES.new(self.pad_key(self.key).encode(), self.mode)

    # 加密函数，如果text不是16的倍数【加密文本text必须为16的倍数！】，那就补足为16的倍数
    # 加密内容需要长达16位字符，所以进行空格拼接
    def pad(self, text):
        while len(text) % self.AES_LENGTH != 0:
            text += ' '
        return text

    # 加密密钥需要长达16位字符，所以进行空格拼接
    def pad_key(self, key):
        while len(key) % self.AES_LENGTH != 0:
            key += ' '
        return key

    def encrypt(self, text):

        # 这里密钥key 长度必须为16（AES-128）、24（AES-192）、或32（AES-256）Bytes 长度.目前AES-128足够用
        # 加密的字符需要转换为bytes
        # print(self.pad(text))
        self.ciphertext = self.cryptor.encrypt(self.pad(text).encode())
        # 因为AES加密时候得到的字符串不一定是ascii字符集的，输出到终端或者保存时候可能存在问题
        # 所以这里统一把加密后的字符串转化为16进制字符串
        return b2a_hex(self.ciphertext)
        # 解密后，去掉补足的空格用strip() 去掉

    def decrypt(self, text):
        plain_text = self.cryptor.decrypt(a2b_hex(text)).decode()
        return plain_text.rstrip(' ')





#!/usr/bin/env python
# -*- encoding: utf-8 -*-

'''
@File    :   netHelper.py
@Time    :   2018/12/17
@Author  :   Yaronzz 
@Version :   2.0
@Contact :   yaronhuang@foxmail.com
@Desc    :   
'''

import re
import os
import sys
import json
import socket
import requests
import shutil
from urllib.request import urlopen
from aigpy.progressHelper import ProgressTool
from aigpy.convertHelper import convertStorageUnit
from aigpy.pathHelper import getFileName, getDirName, mkdirs, getDiffTmpPathName
from aigpy.threadHelper import ThreadTool
from aigpy.fileHelper import getFileContent


def getIpStatus(host, port, timeouts=1):
    socket.setdefaulttimeout(timeouts)
    flag = True
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect((host, port))
        s.close()
    except:
        flag = False
    return flag


def getIP():
    text = requests.get("http://txt.go.sohu.com/ip/soip").text
    ip = re.findall(r'\d+.\d+.\d+.\d+', text)[0]
    return ip


def ignoreCertificate():
    import ssl
    ssl._create_default_https_context = ssl._create_unverified_context


def getResult(code=0, msg='', data=''):
    ret = {}
    ret['code'] = code
    ret['errmsg'] = msg
    ret['data'] = data
    return json.dumps(ret)


def getFileSize(url):
    try:
        response = urlopen(url)
        info = response.info()
        dic = dict(info)
        length = dic['Content-Length']
        return int(length)
    except:
        return -1


def downloadString(url, timeouts=(3.05, 27)):
    try:
        re = requests.get(url, timeouts)
        return re.content
    except:
        return None 
    

def downloadJson(url, timeouts=(3.05, 27)):
    try:
        re = requests.get(url, timeouts)
        info = json.loads(re.content)
        return info
    except:
        return None


def downloadFileByUrls(urlArray, fileName, stimeout=None, showprogress=False):
    if os.access(fileName, 0):
        os.remove(fileName)

    progress = None
    if showprogress:
        desc = getFileName(fileName)
        progress = ProgressTool(len(urlArray), 10, unit='', desc=desc)

    curcount = 1    
    for item in urlArray:
        ret, ex = downloadFile(item, fileName, stimeout, False, append=True)
        if ret != True or ex != None:
            return False
        if progress:
            progress.setCurCount(curcount)
            curcount+=1
    return True


def downloadFile(url, fileName, stimeout=None, showprogress=False, append=False):
    try:
        if stimeout is None:
            response = urlopen(url)
        else:
            response = urlopen(url, timeout=stimeout)
        
        unit = 'mb'
        if convertStorageUnit(response.length, 'byte', unit) < 1:
            unit = 'kb'
        progress = None
        if showprogress:
            desc = ""#getFileName(fileName)
            progress = ProgressTool(convertStorageUnit(response.length, 'byte', unit), 15, unit=unit, desc=desc)

        mode ='wb'
        if append:
            mode = 'ab'

        # mkdir
        path = getDirName(fileName)
        mkdirs(path)

        curcount  = 0
        chunksize = 16 * 1024
        with open(fileName, mode) as f:
            while True:
                chunk = response.read(chunksize)
                curcount += len(chunk)
                if progress:
                    progress.setCurCount(convertStorageUnit(curcount, 'byte', unit))
                if not chunk:
                    break
                f.write(chunk)
            return True, None
    except Exception as e:
        return False, e


def __downloadPartFile__(url, fileName, stimeout=None, start=None, length=None, progress = None, unit = 'mb', retry = 3):
    while retry > 0:
        try:
            headers = None
            if start is not None and length is not None:
                rang = 'bytes=%s-%s' % (start, start + length - 1)
                headers = {'Range': rang}
            
            res = requests.get(url, headers=headers, timeout=stimeout)
            res.raise_for_status()

            # mkdir
            path = getDirName(fileName)
            mkdirs(path)

            with open(fileName, 'wb') as f:
                f.write(res.content)

            if progress is not None:
                progress.addCurCount(convertStorageUnit(length, 'byte', unit))
            return True, None
        except Exception as e:
            if os.path.isfile(fileName):
                os.unlink(fileName)
            if retry <= 0:
                return False, e
            retry -= 1
    return False, "retry num must > 0."


def __mergerPartFiles__(files, filepath):
    try:
        with open(filepath, "wb") as fd:
            for item in files:
                data = getFileContent(item, True)
                fd.write(data)
        return True
    except:
        return False


def downloadFileMultiThread(url, fileName, stimeout=None, showprogress=False, threadnum=30, partsize=1048576):
    try:
        filelength = getFileSize(url)
        if filelength <= 0:
            return False, "File size = " + str(filelength)

        at = 0
       	rangs = []
        length = filelength
        while length > 0:
            if length > partsize:
                bf = [at, partsize]
            else:
                bf = [at, length]
            rangs.append(bf)
            at += partsize
            length -= partsize

        # Creat tmpdir
        path = getDirName(fileName)
        tmpPath = getDiffTmpPathName(path)
        if mkdirs(tmpPath) is False:
            return False, "Creat tmpdir failed:" + tmpPath;

        # thread
        threads = ThreadTool(threadnum)
        # Progress
        unit = 'mb'
        if convertStorageUnit(filelength, 'byte', unit) < 1:
            unit = 'kb'
        progress = None
        if showprogress:
            desc = ""  # getFileName(fileName)
            progress = ProgressTool(convertStorageUnit(filelength, 'byte', unit), 15, unit=unit, desc=desc)

        files = []
        for i, item in enumerate(rangs):
            filepath = tmpPath + '/' + str(i) + '.part'
            files.append(filepath)
            threads.start(__downloadPartFile__, url, filepath, stimeout, item[0], item[1], progress, unit)
        threads.waitAll()

        # merger
        flag = __mergerPartFiles__(files, fileName)
        shutil.rmtree(tmpPath)
        threads.close()
        return flag, None

    except Exception as e:
        shutil.rmtree(tmpPath)
        threads.close()
        return False, e


# import aigpy.convertHelper as AIGConvert
# import aigpy.netHelper as AIGNet
# import aigpy.fileHelper as AIGFile
# import aigpy.stringHelper as AIGString

# fileName = ""
# outfileName = ""
# content = AIGFile.getFileContent(fileName)
# lines = content.split('\n')
# outLines = ""
# for item in lines:
#     size = AIGNet.getFileSize(item)
#     sizeString = AIGConvert.convertStorageUnitToString(size, 'byte')
#     print(sizeString)
#     outLines += '[' + sizeString + '] ' + item + '\n'
# AIGFile.write(outfileName, outLines, 'w+')

